@@ -18,9 +18,7 @@ TextHistory::TextHistory(TextBuffer &buffer)
     clear();
 }
 
-TextHistory::~TextHistory()
-{
-}
+TextHistory::~TextHistory() = default;
 
 qint64 TextHistory::revision() const
 {
@@ -47,7 +45,7 @@ void TextHistory::setLastSavedRevision()
     m_lastSavedRevision = revision();
 }
 
-void TextHistory::wrapLine(const KTextEditor::Cursor &position)
+void TextHistory::wrapLine(const KTextEditor::Cursor position)
 {
     // create and add new entry
     Entry entry;
@@ -68,7 +66,7 @@ void TextHistory::unwrapLine(int line, int oldLineLength)
     addEntry(entry);
 }
 
-void TextHistory::insertText(const KTextEditor::Cursor &position, int length, int oldLineLength)
+void TextHistory::insertText(const KTextEditor::Cursor position, int length, int oldLineLength)
 {
     // create and add new entry
     Entry entry;
@@ -80,7 +78,7 @@ void TextHistory::insertText(const KTextEditor::Cursor &position, int length, in
     addEntry(entry);
 }
 
-void TextHistory::removeText(const KTextEditor::Range &range, int oldLineLength)
+void TextHistory::removeText(KTextEditor::Range range, int oldLineLength)
 {
     // create and add new entry
     Entry entry;
@@ -211,10 +209,11 @@ void TextHistory::Entry::transformCursor(int &cursorLine, int &cursorColumn, boo
         }
 
         // skip cursors with too small column
-        if (cursorColumn <= column)
+        if (cursorColumn <= column) {
             if (cursorColumn < column || !moveOnInsert) {
                 return;
             }
+        }
 
         // patch column of cursor
         if (cursorColumn <= oldLineLength) {
@@ -328,10 +327,11 @@ void TextHistory::Entry::reverseTransformCursor(int &cursorLine, int &cursorColu
         }
 
         // skip cursors with too small column
-        if (cursorColumn <= column)
+        if (cursorColumn <= column) {
             if (cursorColumn < column || !moveOnInsert) {
                 return;
             }
+        }
 
         // patch column of cursor
         if (cursorColumn <= oldLineLength) {
@@ -391,7 +391,11 @@ void TextHistory::transformCursor(int &line, int &column, KTextEditor::MovingCur
     }
 }
 
-void TextHistory::transformRange(KTextEditor::Range &range, KTextEditor::MovingRange::InsertBehaviors insertBehaviors, KTextEditor::MovingRange::EmptyBehavior emptyBehavior, qint64 fromRevision, qint64 toRevision)
+void TextHistory::transformRange(KTextEditor::Range &range,
+                                 KTextEditor::MovingRange::InsertBehaviors insertBehaviors,
+                                 KTextEditor::MovingRange::EmptyBehavior emptyBehavior,
+                                 qint64 fromRevision,
+                                 qint64 toRevision)
 {
     // invalidate on empty?
     bool invalidateIfEmpty = emptyBehavior == KTextEditor::MovingRange::InvalidateIfEmpty;
@@ -425,7 +429,10 @@ void TextHistory::transformRange(KTextEditor::Range &range, KTextEditor::MovingR
     // transform cursors
 
     // first: copy cursors, without range association
-    int startLine = range.start().line(), startColumn = range.start().column(), endLine = range.end().line(), endColumn = range.end().column();
+    int startLine = range.start().line();
+    int startColumn = range.start().column();
+    int endLine = range.end().line();
+    int endColumn = range.end().column();
 
     bool moveOnInsertStart = !(insertBehaviors & KTextEditor::MovingRange::ExpandLeft);
     bool moveOnInsertEnd = (insertBehaviors & KTextEditor::MovingRange::ExpandRight);
