@@ -38,9 +38,6 @@
 
 const bool hideAutomaticCompletionOnExactMatch = true;
 
-// If this is true, the completion-list is navigated up/down when 'tab' is pressed, instead of doing partial completion
-const bool shellLikeTabCompletion = false;
-
 #define CALLCI(WHAT, WHATELSE, WHAT2, model, FUNC)                                                                                                             \
     {                                                                                                                                                          \
         static KTextEditor::CodeCompletionModelControllerInterface defaultIf;                                                                                  \
@@ -111,24 +108,6 @@ KateCompletionWidget::KateCompletionWidget(KTextEditor::ViewPrivate *parent)
     // FIXME: this was imported from the config dialog deactivated years ago
     // without this setup either nothing shows or it segfaults
 
-    // Grouping
-    m_presentationModel->setGroupingEnabled(true);
-
-    m_presentationModel->setGroupingMethod(KateCompletionModel::ScopeType | KateCompletionModel::AccessType);
-
-    m_presentationModel->setAccessIncludeConst(false);
-    m_presentationModel->setAccessIncludeStatic(false);
-    m_presentationModel->setAccessIncludeSignalSlot(false);
-
-    // Column merging
-    m_presentationModel->setColumnMergingEnabled(true);
-
-    QList<QList<int>> mergedColumns;
-    mergedColumns << (QList<int>() << 0);
-    mergedColumns << (QList<int>() << 1 << 2 << 3 << 4);
-    mergedColumns << (QList<int>() << 5);
-    m_presentationModel->setColumnMerges(mergedColumns);
-
     connect(parent, &KTextEditor::ViewPrivate::navigateAccept, this, &KateCompletionWidget::navigateAccept);
     connect(parent, &KTextEditor::ViewPrivate::navigateBack, this, &KateCompletionWidget::navigateBack);
     connect(parent, &KTextEditor::ViewPrivate::navigateDown, this, &KateCompletionWidget::navigateDown);
@@ -197,6 +176,9 @@ KateCompletionWidget::~KateCompletionWidget()
     // ensure no slot triggered during destruction => else we access already invalidated stuff
     m_presentationModel->disconnect(this);
     m_argumentHintModel->disconnect(this);
+
+    delete m_argumentHintTree;
+    m_argumentHintTree = nullptr;
 }
 
 void KateCompletionWidget::viewFocusOut()
@@ -297,9 +279,6 @@ KateCompletionModel *KateCompletionWidget::model()
 void KateCompletionWidget::rowsInserted(const QModelIndex &parent, int rowFrom, int rowEnd)
 {
     m_entryList->setAnimated(false);
-    if (!model()->isGroupingEnabled()) {
-        return;
-    }
 
     if (!parent.isValid()) {
         for (int i = rowFrom; i <= rowEnd; ++i) {
@@ -740,10 +719,14 @@ void KateCompletionWidget::cursorPositionChanged()
         oldCurrentSourceIndex = m_presentationModel->mapToSource(m_entryList->currentIndex());
     }
 
+    QMap<KTextEditor::CodeCompletionModel *, QString> filterStringByModel;
+
+    disconnect(this->model(), &KateCompletionModel::layoutChanged, this, &KateCompletionWidget::modelContentChanged);
+    disconnect(this->model(), &KateCompletionModel::modelReset, this, &KateCompletionWidget::modelContentChanged);
+
     // Check the models and eventually abort some
     const QList<KTextEditor::CodeCompletionModel *> checkCompletionRanges = m_completionRanges.keys();
-    for (QList<KTextEditor::CodeCompletionModel *>::const_iterator it = checkCompletionRanges.begin(); it != checkCompletionRanges.end(); ++it) {
-        KTextEditor::CodeCompletionModel *model = *it;
+    for (auto model : checkCompletionRanges) {
         if (!m_completionRanges.contains(model)) {
             continue;
         }
@@ -799,10 +782,15 @@ void KateCompletionWidget::cursorPositionChanged()
                 m_presentationModel->removeCompletionModel(model);
             }
         } else {
-            m_presentationModel->setCurrentCompletion(model, currentCompletion);
+            filterStringByModel[model] = currentCompletion;
         }
     }
 
+    connect(this->model(), &KateCompletionModel::layoutChanged, this, &KateCompletionWidget::modelContentChanged);
+    connect(this->model(), &KateCompletionModel::modelReset, this, &KateCompletionWidget::modelContentChanged);
+
+    m_presentationModel->setCurrentCompletion(filterStringByModel);
+
     if (oldCurrentSourceIndex.isValid()) {
         QModelIndex idx = m_presentationModel->mapFromSource(oldCurrentSourceIndex);
         // We only want to reselect this if it is still the first item
@@ -945,7 +933,9 @@ bool KateCompletionWidget::execute()
         }
         const auto &multicursors = view()->secondaryCursors();
         for (const auto &c : multicursors) {
-            KTextEditor::Range wordToReplace = view()->doc()->wordRangeAt(c.cursor());
+            const KTextEditor::Cursor pos = c.cursor();
+            KTextEditor::Range wordToReplace = view()->doc()->wordRangeAt(pos);
+            wordToReplace.setEnd(pos); // limit the word to the current cursor position
             view()->doc()->replaceText(wordToReplace, text);
         }
     };
@@ -1378,7 +1368,7 @@ int KateCompletionWidget::automaticInvocationDelay() const
     return m_automaticInvocationDelay;
 }
 
-void KateCompletionWidget::setIgnoreBufferSignals(bool ignore)
+void KateCompletionWidget::setIgnoreBufferSignals(bool ignore) const
 {
     if (ignore) {
         disconnect(&view()->doc()->buffer(), &KateBuffer::lineWrapped, this, &KateCompletionWidget::wrapLine);
@@ -1441,15 +1431,6 @@ void KateCompletionWidget::insertText(const KTextEditor::Cursor &position, const
         return;
     }
 
-    // TODO: Following is disabled because it causes some tests to fail, check
-    // and fix the tests / remove this
-
-    // We are triggering automaticInvocation()
-    // insertText tiggeres cursorPositionChanged()
-    // which will trigger modelContentChanged() indirectly. So we disconnect here
-    //     disconnect(this->model(), &KateCompletionModel::layoutChanged, this, &KateCompletionWidget::modelContentChanged);
-    //     disconnect(this->model(), &KateCompletionModel::modelReset, this, &KateCompletionWidget::modelContentChanged);
-
     m_automaticInvocationTimer->start(m_automaticInvocationDelay);
 }
 
@@ -1498,47 +1479,24 @@ void KateCompletionWidget::userInvokedCompletion()
     startCompletion(KTextEditor::CodeCompletionModel::UserInvocation);
 }
 
-void KateCompletionWidget::tab(bool shift)
+void KateCompletionWidget::tabCompletion(Direction direction)
 {
-    m_noAutoHide = true;
-    if (!m_entryList->currentIndex().isValid()) {
-        // No current selection. To not make the tab key a noop, select the first entry
-        cursorDown();
+    if (!m_inCompletionList) {
         return;
     }
 
-    if (!shift) {
-        QString prefix = m_presentationModel->commonPrefix((m_inCompletionList && !shellLikeTabCompletion) ? m_entryList->currentIndex() : QModelIndex());
-        if (!prefix.isEmpty()) {
-            view()->insertText(prefix);
-        } else if (shellLikeTabCompletion) {
-            cursorDown();
-            return;
-        }
-    } else {
-        if (shellLikeTabCompletion) {
-            cursorUp();
-            return;
-        }
+    m_noAutoHide = true;
 
-        // Reset left boundaries, so completion isn't stopped
-        typedef QMap<KTextEditor::CodeCompletionModel *, CompletionRange> CompletionRangeMap;
-        for (CompletionRangeMap::iterator it = m_completionRanges.begin(); it != m_completionRanges.end(); ++it) {
-            (*it).leftBoundary = (*it).range->start();
+    // Not using cursorDown/Up() as we don't want to go into the argument-hint list
+    if (direction == Down) {
+        const bool res = m_entryList->nextCompletion();
+        if (!res) {
+            m_entryList->top();
         }
-
-        // Remove suffix until the completion-list filter is widened again
-        uint itemCount = m_presentationModel->filteredItemCount();
-
-        while (view()->cursorPosition().column() > 0 && m_presentationModel->filteredItemCount() == itemCount) {
-            KTextEditor::Range lastcharRange = KTextEditor::Range(view()->cursorPosition() - KTextEditor::Cursor(0, 1), view()->cursorPosition());
-            QString cursorText = view()->document()->text(lastcharRange);
-            if (!cursorText[0].isSpace()) {
-                view()->document()->removeText(lastcharRange);
-                QApplication::sendPostedEvents();
-            } else {
-                break;
-            }
+    } else { // direction == Up
+        const bool res = m_entryList->previousCompletion();
+        if (!res) {
+            m_entryList->bottom();
         }
     }
 }
