@@ -68,13 +68,13 @@ class KToggleAction;
  * @warning This file is @e private API and not part of the public
  *          KTextEditor interfaces.
  */
-class KTEXTEDITOR_EXPORT KTextEditor::DocumentPrivate : public KTextEditor::Document,
-                                                        public KTextEditor::MarkInterfaceV2,
-                                                        public KTextEditor::ModificationInterface,
-                                                        public KTextEditor::ConfigInterface,
-                                                        public KTextEditor::AnnotationInterface,
-                                                        public KTextEditor::MovingInterface,
-                                                        private KTextEditor::MovingRangeFeedback
+class KTEXTEDITOR_EXPORT KTextEditor::DocumentPrivate final : public KTextEditor::Document,
+                                                              public KTextEditor::MarkInterfaceV2,
+                                                              public KTextEditor::ModificationInterface,
+                                                              public KTextEditor::ConfigInterface,
+                                                              public KTextEditor::AnnotationInterface,
+                                                              public KTextEditor::MovingInterface,
+                                                              private KTextEditor::MovingRangeFeedback
 {
     Q_OBJECT
     Q_INTERFACES(KTextEditor::MarkInterface)
@@ -391,7 +391,7 @@ public Q_SLOTS:
      * If the consecutive editings happens in the same line, then remove
      * the previous and add the new one with updated column no.
      */
-    void saveEditingPositions(const KTextEditor::Cursor &cursor);
+    void saveEditingPositions(const KTextEditor::Cursor cursor);
 
 public:
     uint undoCount() const;
@@ -409,7 +409,7 @@ Q_SIGNALS:
     void undoChanged();
 
 public:
-    QVector<KTextEditor::Range> searchText(const KTextEditor::Range &range, const QString &pattern, const KTextEditor::SearchOptions options) const;
+    QVector<KTextEditor::Range> searchText(KTextEditor::Range range, const QString &pattern, const KTextEditor::SearchOptions options) const;
 
 private:
     /**
@@ -417,6 +417,14 @@ private:
      */
     QWidget *dialogParent();
 
+    /**
+     * Wrapper around QFileDialog::getSaveFileUrl, will use proper dialogParent
+     * and try it's best to find a good directory as start
+     * @param dialogTitle dialog title string
+     * @return url to save to or empty url if aborted
+     */
+    QUrl getSaveFileUrl(const QString &dialogTitle);
+
     /*
      * Access to the mode/highlighting subsystem
      */
@@ -836,24 +844,40 @@ public:
     bool ownedView(KTextEditor::ViewPrivate *);
 
     int toVirtualColumn(int line, int column) const;
-    int toVirtualColumn(const KTextEditor::Cursor &) const;
+    int toVirtualColumn(const KTextEditor::Cursor) const;
     int fromVirtualColumn(int line, int column) const;
-    int fromVirtualColumn(const KTextEditor::Cursor &) const;
+    int fromVirtualColumn(const KTextEditor::Cursor) const;
 
     enum NewLineIndent { Indent, NoIndent };
+    enum NewLinePos { Normal, Above, Below };
 
-    void newLine(KTextEditor::ViewPrivate *view, NewLineIndent indent = NewLineIndent::Indent); // Changes input
-    void backspace(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor &);
-    void del(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor &);
-    void transpose(const KTextEditor::Cursor &);
+    void newLine(KTextEditor::ViewPrivate *view, NewLineIndent indent = NewLineIndent::Indent, NewLinePos newLinePos = Normal); // Changes input
+    void backspace(KTextEditor::ViewPrivate *view);
+    void del(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor);
+    void transpose(const KTextEditor::Cursor);
     void swapTextRanges(KTextEditor::Range firstWord, KTextEditor::Range secondWord);
+    bool multiPaste(KTextEditor::ViewPrivate *view, const QStringList &texts);
     void paste(KTextEditor::ViewPrivate *view, const QString &text);
 
+public:
+    enum CommentType {
+        UnComment = -1,
+        ToggleComment = 0,
+        Comment = 1,
+    };
+
+private:
+    // Helper function for use with multiple cursors
+    KTextEditor::Cursor backspaceAtCursor(KTextEditor::ViewPrivate *v, KTextEditor::Cursor c);
+    void commentSelection(KTextEditor::Range selection, KTextEditor::Cursor c, bool blockSelect, CommentType changeType);
+    bool skipAutoBrace(QChar closingBracket, KTextEditor::Cursor pos);
+
 public:
     void indent(KTextEditor::Range range, int change);
-    void comment(KTextEditor::ViewPrivate *view, uint line, uint column, int change);
-    void align(KTextEditor::ViewPrivate *view, const KTextEditor::Range &range);
-    void insertTab(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor &);
+    void comment(KTextEditor::ViewPrivate *view, uint line, uint column, CommentType change);
+    void align(KTextEditor::ViewPrivate *view, KTextEditor::Range range);
+    void alignOn(KTextEditor::Range range, const QString &pattern, bool blockwise);
+    void insertTab(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor);
 
     enum TextTransform { Uppercase, Lowercase, Capitalize };
 
@@ -864,13 +888,15 @@ public:
       lowercase the character right of the cursor is transformed, for capitalize
       the word under the cursor is transformed.
     */
-    void transform(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor &, TextTransform);
+    void transform(KTextEditor::ViewPrivate *view, KTextEditor::Cursor, TextTransform);
     /**
       Unwrap a range of lines.
     */
     void joinLines(uint first, uint last);
 
 private:
+    void transformCursorOrRange(KTextEditor::ViewPrivate *view, const KTextEditor::Cursor, KTextEditor::Range selection, TextTransform);
+
     bool removeStringFromBeginning(int line, const QString &str);
     bool removeStringFromEnd(int line, const QString &str);
 
@@ -879,7 +905,7 @@ private:
       @param cursorPos The current cursor position for the inserted characters.
       @param str The typed characters to expand.
     */
-    QString eventuallyReplaceTabs(const KTextEditor::Cursor &cursorPos, const QString &str) const;
+    QString eventuallyReplaceTabs(const KTextEditor::Cursor cursorPos, const QString &str) const;
 
     /**
       Find the position (line and col) of the next char
@@ -923,17 +949,17 @@ private:
     /**
      *@see removeStartLineCommentFromSingleLine.
      */
-    bool removeStartStopCommentFromRegion(const KTextEditor::Cursor &start, const KTextEditor::Cursor &end, int attrib = 0);
+    bool removeStartStopCommentFromRegion(const KTextEditor::Cursor start, const KTextEditor::Cursor end, int attrib = 0);
 
     /**
      * Add a comment marker as defined by the language providing the attribute
      * @p attrib to each line in the selection.
      */
-    void addStartStopCommentToSelection(KTextEditor::ViewPrivate *view, int attrib = 0);
+    void addStartStopCommentToSelection(KTextEditor::Range, bool blockSelection, int attrib = 0);
     /**
      * @see addStartStopCommentToSelection.
      */
-    void addStartLineCommentToSelection(KTextEditor::ViewPrivate *view, int attrib = 0);
+    void addStartLineCommentToSelection(KTextEditor::Range, int attrib = 0);
 
     /**
      * Removes comment markers relevant to the language providing
@@ -941,14 +967,14 @@ private:
      *
      * @return whether the operation succeeded.
      */
-    bool removeStartStopCommentFromSelection(KTextEditor::ViewPrivate *view, int attrib = 0);
+    bool removeStartStopCommentFromSelection(KTextEditor::Range, int attrib = 0);
     /**
      * @see removeStartStopCommentFromSelection.
      */
-    bool removeStartLineCommentFromSelection(KTextEditor::ViewPrivate *view, int attrib = 0);
+    bool removeStartLineCommentFromSelection(KTextEditor::Range, int attrib, bool toggleComment);
 
 public:
-    KTextEditor::Range findMatchingBracket(const KTextEditor::Cursor &start, int maxLines);
+    KTextEditor::Range findMatchingBracket(const KTextEditor::Cursor start, int maxLines);
 
 public:
     QString documentName() const override
@@ -958,6 +984,7 @@ public:
 
 private:
     void updateDocName();
+    static void uniquifyDocNames(const std::vector<KTextEditor::DocumentPrivate *> &docs);
 
 public:
     /**
@@ -1044,7 +1071,7 @@ public:
     Kate::TextLine plainKateTextLine(int i);
 
 Q_SIGNALS:
-    void aboutToRemoveText(const KTextEditor::Range &);
+    void aboutToRemoveText(KTextEditor::Range);
 
 private Q_SLOTS:
     void slotModOnHdDirty(const QString &path);
@@ -1067,9 +1094,10 @@ private:
     QString reasonedMOHString() const;
 
     /**
-     * Removes all trailing whitespace in the document.
+     * Removes all trailing whitespace in the document and add new line at eof
+     * if configured that way.
      */
-    void removeTrailingSpaces();
+    void removeTrailingSpacesAndAddNewLineAtEof();
 
 public:
     /**
@@ -1259,37 +1287,41 @@ public:
     QString defaultDictionary() const;
     QList<QPair<KTextEditor::MovingRange *, QString>> dictionaryRanges() const;
     bool isOnTheFlySpellCheckingEnabled() const;
+    KateOnTheFlyChecker *onTheFlySpellChecker() const
+    {
+        return m_onTheFlyChecker;
+    }
 
-    QString dictionaryForMisspelledRange(const KTextEditor::Range &range) const;
+    QString dictionaryForMisspelledRange(KTextEditor::Range range) const;
     void clearMisspellingForWord(const QString &word);
 
 public Q_SLOTS:
     void clearDictionaryRanges();
-    void setDictionary(const QString &dict, const KTextEditor::Range &range, bool blockmode);
-    void setDictionary(const QString &dict, const KTextEditor::Range &range);
+    void setDictionary(const QString &dict, KTextEditor::Range range, bool blockmode);
+    void setDictionary(const QString &dict, KTextEditor::Range range);
     void setDefaultDictionary(const QString &dict);
     void onTheFlySpellCheckingEnabled(bool enable);
-    void refreshOnTheFlyCheck(const KTextEditor::Range &range = KTextEditor::Range::invalid());
+    void refreshOnTheFlyCheck(KTextEditor::Range range = KTextEditor::Range::invalid());
 
 Q_SIGNALS:
     void dictionaryRangesPresent(bool yesNo);
     void defaultDictionaryChanged(KTextEditor::DocumentPrivate *document);
 
 public:
-    bool containsCharacterEncoding(const KTextEditor::Range &range);
+    bool containsCharacterEncoding(KTextEditor::Range range);
 
     typedef QList<QPair<int, int>> OffsetList;
 
-    int computePositionWrtOffsets(const OffsetList &offsetList, int pos);
+    static int computePositionWrtOffsets(const OffsetList &offsetList, int pos);
 
     /**
      * The first OffsetList is from decoded to encoded, and the second OffsetList from
      * encoded to decoded.
      **/
-    QString decodeCharacters(const KTextEditor::Range &range,
+    QString decodeCharacters(KTextEditor::Range range,
                              KTextEditor::DocumentPrivate::OffsetList &decToEncOffsetList,
                              KTextEditor::DocumentPrivate::OffsetList &encToDecOffsetList);
-    void replaceCharactersByEncoding(const KTextEditor::Range &range);
+    void replaceCharactersByEncoding(KTextEditor::Range range);
 
 protected:
     KateOnTheFlyChecker *m_onTheFlyChecker = nullptr;
@@ -1438,7 +1470,7 @@ private:
     QChar m_currentAutobraceClosingChar;
 
 private Q_SLOTS:
-    void checkCursorForAutobrace(KTextEditor::View *view, const KTextEditor::Cursor &newPos);
+    void checkCursorForAutobrace(KTextEditor::View *view, const KTextEditor::Cursor newPos);
 
 public:
     void setActiveTemplateHandler(KateTemplateHandler *handler);
@@ -1450,6 +1482,8 @@ private:
     // To calculate a QHash.keys() is quite expensive,
     // better keep a copy of that list updated when a view is added or removed.
     QList<KTextEditor::View *> m_viewsCache;
+
+    QTimer m_autoSaveTimer;
 };
 
 #endif
