@@ -14,16 +14,16 @@
 
 #include <KColorButton>
 #include <KConfigGroup>
-#include <KLineEdit>
+#include <KFontRequester>
 #include <KLocalizedString>
 #include <KSharedConfig>
 #include <QComboBox>
 
 #include <QBoxLayout>
 #include <QCheckBox>
-#include <QFontDialog>
 #include <QGroupBox>
 #include <QLabel>
+#include <QLineEdit>
 #include <QMenu>
 #include <QSpinBox>
 
@@ -118,13 +118,8 @@ KatePrintHeaderFooter::KatePrintHeaderFooter(QWidget *parent)
     QHBoxLayout *lo2 = new QHBoxLayout();
     lo->addLayout(lo2);
     lo2->addWidget(new QLabel(i18n("Header/footer font:"), this));
-    lFontPreview = new QLabel(this);
-    lFontPreview->setFrameStyle(QFrame::Panel | QFrame::Sunken);
+    lFontPreview = new KFontRequester(this);
     lo2->addWidget(lFontPreview);
-    lo2->setStretchFactor(lFontPreview, 1);
-    QPushButton *btnChooseFont = new QPushButton(i18n("Choo&se Font..."), this);
-    lo2->addWidget(btnChooseFont);
-    connect(btnChooseFont, &QPushButton::clicked, this, &KatePrintHeaderFooter::setHFFont);
 
     // header
     gbHeader = new QGroupBox(this);
@@ -139,20 +134,20 @@ KatePrintHeaderFooter::KatePrintHeaderFooter(QWidget *parent)
     QHBoxLayout *layoutFormat = new QHBoxLayout(hbHeaderFormat);
     grid->addWidget(hbHeaderFormat, 0, 1);
 
-    leHeaderLeft = new KLineEdit(hbHeaderFormat);
+    leHeaderLeft = new QLineEdit(hbHeaderFormat);
     layoutFormat->addWidget(leHeaderLeft);
-    leHeaderCenter = new KLineEdit(hbHeaderFormat);
+    leHeaderCenter = new QLineEdit(hbHeaderFormat);
     layoutFormat->addWidget(leHeaderCenter);
-    leHeaderRight = new KLineEdit(hbHeaderFormat);
+    leHeaderRight = new QLineEdit(hbHeaderFormat);
     lHeaderFormat->setBuddy(leHeaderLeft);
     layoutFormat->addWidget(leHeaderRight);
 
     leHeaderLeft->setContextMenuPolicy(Qt::CustomContextMenu);
     leHeaderCenter->setContextMenuPolicy(Qt::CustomContextMenu);
     leHeaderRight->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(leHeaderLeft, &KLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
-    connect(leHeaderCenter, &KLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
-    connect(leHeaderRight, &KLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
+    connect(leHeaderLeft, &QLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
+    connect(leHeaderCenter, &QLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
+    connect(leHeaderRight, &QLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
 
     grid->addWidget(new QLabel(i18n("Colors:"), gbHeader), 1, 0);
 
@@ -185,20 +180,20 @@ KatePrintHeaderFooter::KatePrintHeaderFooter(QWidget *parent)
     layoutFormat->setSpacing(-1);
     grid->addWidget(hbFooterFormat, 0, 1);
 
-    leFooterLeft = new KLineEdit(hbFooterFormat);
+    leFooterLeft = new QLineEdit(hbFooterFormat);
     layoutFormat->addWidget(leFooterLeft);
-    leFooterCenter = new KLineEdit(hbFooterFormat);
+    leFooterCenter = new QLineEdit(hbFooterFormat);
     layoutFormat->addWidget(leFooterCenter);
-    leFooterRight = new KLineEdit(hbFooterFormat);
+    leFooterRight = new QLineEdit(hbFooterFormat);
     layoutFormat->addWidget(leFooterRight);
     lFooterFormat->setBuddy(leFooterLeft);
 
     leFooterLeft->setContextMenuPolicy(Qt::CustomContextMenu);
     leFooterCenter->setContextMenuPolicy(Qt::CustomContextMenu);
     leFooterRight->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(leFooterLeft, &KLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
-    connect(leFooterCenter, &KLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
-    connect(leFooterRight, &KLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
+    connect(leFooterLeft, &QLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
+    connect(leFooterCenter, &QLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
+    connect(leFooterRight, &QLineEdit::customContextMenuRequested, this, &KatePrintHeaderFooter::showContextMenu);
 
     grid->addWidget(new QLabel(i18n("Colors:"), gbFooter), 1, 0);
 
@@ -329,16 +324,6 @@ bool KatePrintHeaderFooter::useFooterBackground()
     return cbFooterEnableBgColor->isChecked();
 }
 
-void KatePrintHeaderFooter::setHFFont()
-{
-    bool ok = false;
-    const QFont selectedFont = QFontDialog::getFont(&ok, lFontPreview->font(), this);
-    if (ok) {
-        lFontPreview->setFont(selectedFont);
-        lFontPreview->setText(i18n("%1, %2pt", selectedFont.family(), selectedFont.pointSize()));
-    }
-}
-
 void KatePrintHeaderFooter::showContextMenu(const QPoint &pos)
 {
     QLineEdit *lineEdit = qobject_cast<QLineEdit *>(sender());
@@ -435,10 +420,7 @@ void KatePrintHeaderFooter::readSettings()
     QColor footerBackground = headerFooterGroup.readEntry("FooterBackground", QColor("lightgrey"));
     kcbtnFooterBg->setColor(footerBackground);
 
-    // Font
-    QFont headerFooterFont = headerFooterGroup.readEntry("HeaderFooterFont", QFont());
-    lFontPreview->setFont(headerFooterFont);
-    lFontPreview->setText(QString(headerFooterFont.family() + QLatin1String(", %1pt")).arg(headerFooterFont.pointSize()));
+    lFontPreview->setFont(headerFooterGroup.readEntry("HeaderFooterFont", KTextEditor::Editor::instance()->font()));
 }
 
 void KatePrintHeaderFooter::writeSettings()
@@ -495,6 +477,13 @@ KatePrintLayout::KatePrintLayout(QWidget *parent)
     cmbSchema->setEditable(false);
     lSchema->setBuddy(cmbSchema);
 
+    // font
+    QHBoxLayout *lo2 = new QHBoxLayout();
+    lo->addLayout(lo2);
+    lo2->addWidget(new QLabel(i18n("Font:"), this));
+    lFontPreview = new KFontRequester(this);
+    lo2->addWidget(lFontPreview);
+
     cbDrawBackground = new QCheckBox(i18n("Draw bac&kground color"), this);
     lo->addWidget(cbDrawBackground);
 
@@ -564,6 +553,11 @@ KatePrintLayout::~KatePrintLayout()
     writeSettings();
 }
 
+QFont KatePrintLayout::textFont()
+{
+    return lFontPreview->font();
+}
+
 QString KatePrintLayout::colorScheme()
 {
     return cmbSchema->itemData(cmbSchema->currentIndex()).toString();
@@ -608,6 +602,9 @@ void KatePrintLayout::readSettings()
         cmbSchema->setCurrentIndex(index);
     }
 
+    // Font
+    lFontPreview->setFont(layoutGroup.readEntry("Font", KTextEditor::Editor::instance()->font()));
+
     bool isBackgroundChecked = layoutGroup.readEntry("BackgroundColorEnabled", false);
     cbDrawBackground->setChecked(isBackgroundChecked);
 
@@ -631,6 +628,7 @@ void KatePrintLayout::writeSettings()
 
     KConfigGroup layoutGroup(&printGroup, "Layout");
     layoutGroup.writeEntry("ColorScheme", colorScheme());
+    layoutGroup.writeEntry("Font", textFont());
     layoutGroup.writeEntry("BackgroundColorEnabled", useBackground());
     layoutGroup.writeEntry("BoxEnabled", useBox());
     layoutGroup.writeEntry("BoxWidth", boxWidth());
