@@ -12,10 +12,12 @@
 
 #include <QWidget>
 
+class KColorButton;
+class KFontRequester;
+
 class QCheckBox;
 class QLabel;
-class KLineEdit;
-class KColorButton;
+class QLineEdit;
 class QComboBox;
 class QSpinBox;
 class QGroupBox;
@@ -33,7 +35,7 @@ class KatePrintTextSettings : public QWidget
     Q_OBJECT
 public:
     explicit KatePrintTextSettings(QWidget *parent = nullptr);
-    ~KatePrintTextSettings();
+    ~KatePrintTextSettings() override;
 
     bool printLineNumbers();
     bool printGuide();
@@ -61,7 +63,7 @@ class KatePrintHeaderFooter : public QWidget
     Q_OBJECT
 public:
     explicit KatePrintHeaderFooter(QWidget *parent = nullptr);
-    ~KatePrintHeaderFooter();
+    ~KatePrintHeaderFooter() override;
 
     QFont font();
 
@@ -78,7 +80,6 @@ public:
     bool useFooterBackground();
 
 public Q_SLOTS:
-    void setHFFont();
     void showContextMenu(const QPoint &pos);
 
 private:
@@ -86,12 +87,12 @@ private:
     void writeSettings();
 
     QCheckBox *cbEnableHeader, *cbEnableFooter;
-    QLabel *lFontPreview;
+    KFontRequester *lFontPreview;
     QGroupBox *gbHeader, *gbFooter;
-    KLineEdit *leHeaderLeft, *leHeaderCenter, *leHeaderRight;
+    QLineEdit *leHeaderLeft, *leHeaderCenter, *leHeaderRight;
     KColorButton *kcbtnHeaderFg, *kcbtnHeaderBg;
     QCheckBox *cbHeaderEnableBgColor;
-    KLineEdit *leFooterLeft, *leFooterCenter, *leFooterRight;
+    QLineEdit *leFooterLeft, *leFooterCenter, *leFooterRight;
     KColorButton *kcbtnFooterFg, *kcbtnFooterBg;
     QCheckBox *cbFooterEnableBgColor;
 };
@@ -113,9 +114,10 @@ class KatePrintLayout : public QWidget
     Q_OBJECT
 public:
     explicit KatePrintLayout(QWidget *parent = nullptr);
-    ~KatePrintLayout();
+    ~KatePrintLayout() override;
 
     QString colorScheme();
+    QFont textFont();
     bool useBackground();
     bool useBox();
     int boxWidth();
@@ -127,6 +129,7 @@ private:
     void writeSettings();
 
     QComboBox *cmbSchema;
+    KFontRequester *lFontPreview;
     QCheckBox *cbEnableBox;
     QCheckBox *cbDrawBackground;
     QGroupBox *gbBoxProps;
