@@ -233,13 +233,19 @@ public:
         {
             return l.cursor() == r.cursor();
         }
+
+        void clearSelection()
+        {
+            range.reset();
+            anchor = KTextEditor::Cursor::invalid();
+        }
     };
 
     // Just a helper to control the states in which we disallow multicursor
     bool isMulticursorNotAllowed() const;
 
     // Adds a secondary cursor
-    bool addSecondaryCursorAt(const KTextEditor::Cursor &cursor, bool toggle = true);
+    void addSecondaryCursor(const KTextEditor::Cursor &cursor);
     void setSecondaryCursors(const QVector<KTextEditor::Cursor> &positions);
 
     const std::vector<SecondaryCursor> &secondaryCursors() const;
@@ -253,11 +259,24 @@ public:
     // if @p matchLine is true, cursors whose line are same will be considered equal
     void ensureUniqueCursors(bool matchLine = false);
 
+    // For multicursor external api
+    QVector<KTextEditor::Cursor> cursors() const;
+    QVector<KTextEditor::Range> selectionRanges() const;
+
+    void setCursors(const QVector<KTextEditor::Cursor> &cursorPositions);
+    void setSelections(const QVector<KTextEditor::Range> &selectionRanges);
+
+    // Returns true if primary or secondary cursors have selection
+    bool hasSelections() const;
+
 private:
-    void removeSecondaryCursors(const std::vector<KTextEditor::Cursor> &cursorToRemove);
+    bool removeSecondaryCursors(const std::vector<KTextEditor::Cursor> &cursorToRemove, bool removeIfOverlapsSelection = false);
     Kate::TextRange *newSecondarySelectionRange(KTextEditor::Range);
+    void sortCursors();
+    void paintCursors();
 
     std::vector<SecondaryCursor> m_secondaryCursors;
+    bool m_skipCurrentSelection = false;
 
     void addSecondaryCursorDown();
     void addSecondaryCursorUp();
@@ -529,7 +548,9 @@ public Q_SLOTS:
     void indent();
     void unIndent();
     void cleanIndent();
-    void align();
+    void formatIndent();
+    void align(); // alias of formatIndent, for backward compatibility
+    void alignOn();
     void comment();
     void uncomment();
     void toggleComment();
@@ -712,11 +733,13 @@ public Q_SLOTS:
     void findSelectedBackwards();
     void findNextOccurunceAndSelect();
     void findAllOccuruncesAndSelect();
+    void skipCurrentOccurunceSelection();
     void replace();
     void findNext();
     void findPrevious();
     void showSearchWrappedHint(bool isReverseSearch);
     void createMultiCursorsFromSelection();
+    void removeCursorsFromEmptyLines();
 
     void setFoldingMarkersOn(bool enable); // Not in KTextEditor::View, but should be
     void setIconBorder(bool enable);
@@ -736,6 +759,7 @@ public Q_SLOTS:
 
 public:
     int getEol() const;
+    QMenu *getEolMenu();
 
 public:
     KateRenderer *renderer();
@@ -781,7 +805,6 @@ public Q_SLOTS:
     void toggleWriteLock();
     void switchToCmdLine();
     void slotReadWriteChanged();
-    void slotClipboardHistoryChanged();
     void toggleCamelCaseCursor();
 
 Q_SIGNALS:
@@ -828,7 +851,6 @@ private:
     std::vector<QAction *> m_editActions;
     QAction *m_editUndo;
     QAction *m_editRedo;
-    QAction *m_pasteMenu;
     bool m_gotoBottomAfterReload;
     KToggleAction *m_toggleFoldingMarkers;
     KToggleAction *m_toggleIconBar;
@@ -851,6 +873,7 @@ private:
     QAction *m_copy;
     QAction *m_copyHtmlAction;
     QAction *m_paste;
+    QAction *m_clipboardHistory;
     // always nullptr if paste selection isn't supported
     QAction *m_pasteSelection = nullptr;
     QAction *m_swapWithClipboard;
@@ -977,6 +1000,13 @@ private Q_SLOTS:
      */
     void applyFoldingState();
 
+public:
+    /**
+     * Clear any saved folding state
+     */
+    void clearFoldingState();
+
+private:
     void clearHighlights();
     void createHighlights();
 
